//
// CDROM.H: CD-ROM-SDK fr WATCOM C32          (c) in 1996 by FUTUREWARE 2001
//

#define U1 unsigned char
#define S1 char
#define U2 unsigned short
#define S2 short
#ifdef __386__
#define U4 unsigned int
#define S4 int
#define U8 unsigned long
#define S8 long
#endif
#ifndef __386__
#define U4 unsigned long
#define S4 long
#endif

//Schnittstelle:
S1 CD_Init(); // Mu am Anfang aufgerufen werden, Return: 0: Keine CD, 1:CD Ok
void CD_Ende(); // Mu vor ProgrammEnde aufgerufen werden, stoppt die CD.
void CD_Update(); // Sollte in Eingabe-pollings aufgerufen werden
void Play_Musik(U1 Number); // ndert den Musikstil
void Play_Effekt(U1 Number,S1 Stay); // Spielt einen Effekt ab

S1 Music[]={1,2,3,4,5,6,7,8};

//Strukturen
typedef struct
{
  S1 Dummy;
  S2 Offset;
  S2 Segment;
  S1 Daten[0x13];
} IOCTL;
typedef struct
{
  U1 AdressMode;
  U4 Start;
  U4 Laenge;
} PLAYCTL;
typedef union
{
  IOCTL IOCTL;
  PLAYCTL PLAYCTL;
} CDDatentyp;
typedef struct
{
  S1 length;
  S1 Drive;
  S1 Command;
  U2 Status;
  S1 Reserved[8];
  CDDatentyp Daten;
} Struktur1;

typedef struct
{
  S1 Subcommand; // 1
  S1 Format;
  U4 Position;
} GetLesekopf;
typedef struct
{
  S1 Subcommand; // 4
  S1 Info[8];
} GetAudioKanal;
typedef struct
{
  S1 Subcommand; // 6
  U4 Status;
} GetLS;
typedef struct
{
  S1 Subcommand; // 8
  U4 VolumeSize;
} GetVS;
typedef struct
{
  S1 Subcommand; // 9
  S1 Status;
} GetMediaChanged;
typedef struct
{
  S1 Subcommand; // 10
  S1 FirstTrack;
  S1 LastTrack;
  S4 LeadOut; //RB
} GetAudioCD;
typedef struct
{
  S1 Subcommand; // 11
  S1 Track;
  S4 Start;      //Red-Book
  S1 Status;
} GetAudioTrack;
typedef struct
{
  S1 Subcommand; // 14
  S1 Control;
  S1 EAN[7];
} GetUPC;
typedef union
{
  S1 Subcommand;
  GetMediaChanged GMC;
  GetAudioTrack GAT;
  GetAudioKanal GAK;
  GetAudioCD GAC;
  GetUPC UPC;
  GetLS GLS;
  GetLesekopf GLK;
  GetVS GVS;
} Struktur2;

static struct rminfo {
  S8 EDI,ESI,EBP,reserved_by_system,EBX,EDX,ECX,EAX;
  U2 flags,ES,DS,FS,GS,IP,CS,SP,SS;
} RMI;

#define Puffersize 0x30

U4 CDLength;
U4 CDTrackStart[50];
U4 CDTrackLength[50];
U4 CDPosition;
S1 CDEffekt;
U2 CDTracks=0;
U4 CDStatus=0;
S1 CD_Avail=0;
S1 CD_Drive=0;
S1 CD_Music=0;
U2 Puffersegment1,Puffersegment2;
Struktur1 *Puffer1;
Struktur2 *Puffer2;

//***Interne Funktionen
void CD_Reset() // Fhrt einen Treiber-Reset durch,
                // um die Daten auf den neuesten Stand zu bringen!
{
  union REGS regs;
  regs.x.ax=0x1500;
  regs.x.bx=0;
  int386(0x2f,&regs,&regs);
}

S2 Puffer_Init(int size) //Die Real-Mode Puffer werden initialisiert
{
  union REGS regs;
  struct SREGS sregs;

  memset(&sregs,0,sizeof(sregs));
  regs.x.ax=0x0100;
  regs.x.bx=(size+15)>>4;
  int386x( 0x31, &regs, &regs, &sregs);
  memset((void *)(regs.x.ax<<4),0,Puffersize);
  return regs.x.ax;
}

U2 CD_Do() //Zum Ausfhren eines Commands
{
  union REGS regs;
  struct SREGS sregs;

  memset(&RMI,0,sizeof(RMI));
  memset(&sregs,0,sizeof(sregs));
  Puffer1->length=8;
  Puffer1->Drive=CD_Drive;
  RMI.EAX=0x1510;
  RMI.ES=Puffersegment1;
  RMI.ECX=CD_Drive;
  RMI.EBX=0;
  regs.x.ax = 0x0300;
  regs.x.bx = 0x2f;
  regs.x.cx = 0;
  sregs.es = FP_SEG(&RMI);
  regs.d.edi = FP_OFF(&RMI);
  int386x( 0x31, &regs, &regs, &sregs );
  return Puffer1->Status;
}
void IOCTLINIT(S1 Command) // Initialisiert die Puffer fr eine Sub-Funktion
{
  Puffer1->Command=0x3;
  Puffer1->Daten.IOCTL.Offset= 0x00;
  Puffer1->Daten.IOCTL.Segment=Puffersegment2;
  Puffer2->Subcommand=Command;
}

void CD_Stop() //Stoppt die CD
{
  Puffer1->Command=0x85;
  CD_Do();
}

void CD_Play(int Start,int Laenge) //Startet das Abspielen
{
  CD_Stop();
  Puffer1->Command=0x84;
  Puffer1->Daten.PLAYCTL.AdressMode=0;
  Puffer1->Daten.PLAYCTL.Start=Start;
  Puffer1->Daten.PLAYCTL.Laenge=Laenge;
  CD_Do();
}

void CD_Resume()
{
  Puffer1->Command=0x88;
  CD_Do();
}

U4 R2H(U4 RedBook)
{
  U2 Frame = ( RedBook >>  0 ) & 0x00FF;
  U2 Sec   = ( RedBook >>  8 ) & 0x00FF;
  U2 Min   = ( RedBook >> 16 ) & 0x00FF;
  return Min*60L*75L + Sec*75L + Frame - 150;
}

void CD_GetInfo() //Holt alle wichtigen Infos
{
  int i;
  delay(300);
  IOCTLINIT(6);
  CD_Do();
  CDStatus=Puffer2->GLS.Status;
  //printf("Status: %8lx\n",CDStatus);
  if (CDStatus&0x800)
  {
    //printf("Keine CD\n");
    return; //Ist eine CD eingelegt???
  }
  IOCTLINIT(10);
  CD_Do();
  delay(300);
  IOCTLINIT(10);
  CD_Do();
  if (Puffer2->GAC.FirstTrack!=1) return;
  //printf("Erster Track: %d\n",Puffer2->GAC.FirstTrack);
  //printf("Letzter Track: %d\n",Puffer2->GAC.LastTrack);
  CDTracks=Puffer2->GAC.LastTrack;
  CDLength=Puffer2->GAC.LeadOut;
  //printf("Lead Out: %d\n",CDLength);
  IOCTLINIT(8);
  CD_Do();
  CDLength=Puffer2->GVS.VolumeSize;
  //printf("Volume Size: %d\n",CDLength);
  for (i=1;i<=CDTracks;i++)
  {
    IOCTLINIT(11);
    Puffer2->GAT.Track=i;
    CD_Do();
    CDTrackStart[i]=R2H(Puffer2->GAT.Start);
  }
  for(i=1;i<CDTracks;i++)
    CDTrackLength[i]=CDTrackStart[i+1]-CDTrackStart[i];
  CDTrackLength[CDTracks]=CDLength-150-CDTrackStart[CDTracks];

  #ifdef DEBUG
  for (i=1;i<=CDTracks;i++)
  {
    printf("%2d",i);
    printf("Start : %6d",CDTrackStart[i]);
    printf("-%6X",CDTrackStart[i]);
    printf("%2d",i);
    printf("Length: %6d",CDTrackLength[i]);
    printf("-%6X\n",CDTrackLength[i]);
  }
  #endif
  if (CD_Music>CDTracks) CD_Music=0;
}
S2 CD_Playing()
{
  IOCTLINIT(6);
  CD_Do();
  return ((Puffer1->Status&0x0200)==0x0200);
}

//***Externe Funktionen
S1 CD_Init() //Mu am Anfang aufgerufen werden, Return: 0: Keine CD, 1:CD Ok
{
  union REGS regs;

  regs.x.ax=0x1500;
  regs.x.bx=0;
  int386(0x2f,&regs,&regs);
  if(!regs.x.bx) return (0);
  Puffersegment1=Puffer_Init(Puffersize);
  Puffer1=(Struktur1 *)(Puffersegment1<<4);
  Puffersegment2=Puffer_Init(Puffersize);
  Puffer2=(Struktur2 *)(Puffersegment2<<4);
  for (CD_Drive=0;CD_Drive<26;CD_Drive++)
  {
    regs.x.ax=0x150B;
    regs.x.cx=CD_Drive;
    int386(0x2f,&regs,&regs);
    if(regs.x.ax) break;
  }
  if (CD_Drive>=26) return (0);
  #ifdef DEBUG
    printf("Drive %c:\n",CD_Drive+'A');
  #endif
  CD_Avail=1;
  CD_GetInfo();
  CDEffekt=0;
  return (1);
}

void CD_Update() // Sollte in Eingabe-pollings aufgerufen werden
{
  IOCTLINIT(9);
  CD_Do();
  switch(Puffer2->GMC.Status)
  {
    case 0: //Unbekannt
    case -1: //CD wurde gewechselt
      //printf("CD wurde gewechselt.\n");
      CD_Reset();
      CD_GetInfo();
      CD_Play(CDTrackStart[CD_Music],CDTrackLength[CD_Music]);
    break;
    case 1: break; //Kein Wechsel
  };
  if (!CD_Playing())
  {
    if (CDEffekt)
      CD_Play(CDPosition,CDTrackLength[CD_Music]+CDTrackStart[CD_Music]-CDPosition);
    else
      CD_Play(CDTrackStart[CD_Music],CDTrackLength[CD_Music]);
    CDEffekt=0;
  }
  IOCTLINIT(1);
  CD_Do();
  if (!CDEffekt)
  {
    if (Puffer2->GLK.Format==1)
      CDPosition=R2H(Puffer2->GLK.Position);
    else
      CDPosition=Puffer2->GLK.Position;
  }
}

void CD_Ende() // Mu vor ProgrammEnde aufgerufen werden, stoppt die CD.
{
  CD_Stop();
}

void Play_Musik(U1 Number) // ndert den Musikstil
{
  if (Music[Number]==CD_Music)
    return;
  CD_Music=Music[Number];
  CD_Play(CDTrackStart[CD_Music],CDTrackLength[CD_Music]);
}
void Play_Effekt(U1 Number,S1 Stay) //Spielt einen Effekt ab
{
  S1 Ende=0;
  CD_Play(CDTrackStart[Number],CDTrackLength[Number]);
  CDEffekt=1;
  if (Stay)
  {
    while(CD_Playing()&&!Ende)
    {
      if (kbhit()) switch(getch())
      {
        case 27:
          Ende=1;
        break;
      }
    }
    CD_Stop();
  }
}
